=head1 NAME

DynGig::Schedule - Scheduling tool based on DynGig::Range::Time

=cut
package DynGig::Schedule;

=head1 VERSION

Version 0.02

=cut
our $VERSION = '0.02';

=head1 MODULES

=head2 DynGig::Schedule::Policy

Parse schedule

=head2 DynGig::Schedule::Override

Parse override policy

=head2 DynGig::Schedule::Period

Calculate schedule according to a config file

=head1 AUTHOR

Kan Liu

=head1 COPYRIGHT and LICENSE

Copyright (c) 2010. Kan Liu

This program is free software; you may redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;

__END__
